/***********************************************************************************
 * mp3info.cpp
 * Ryan Gilfether <hotrodder@rocketmail.com>
 * Apr 25, 2001
 *
 * Reads the first four bytes of an mp3 file and extracts all header information
 * View the README file included in the distribution for more information
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA (or visit
 * their web site at http://www.gnu.org/).
 ***********************************************************************************/

#include "mp3info.h"

// pass in the full path
mp3info::mp3info(char* str)
{
        file = new ifstream(str, ios::in|ios::binary);
        opened_file = true;
        initialize();
}

// for an already opened file
mp3info::mp3info(ifstream* f)
{
        file = f;
        opened_file = false;
        initialize();
}

mp3info::~mp3info()
{
        if(opened_file)
        {
	        file->close();
                delete file;
        }
}

// returns the emphasis
char* mp3info::emphasis()
{
	char* s;

	switch(bits["Emphasis"])
	{
	case 0:
            s = "None";
	        break;
	case 1:
	        s = "50/15ms";
	        break;
	case 2:
	        s = "Unknown";
	        break;
	case 3:
	        s = "CCITT j.17";
	        break;
	default:
	        s = "Unknown";
	}

	return s;
}

// returns the length in seconds
int mp3info::length()
{
	int br = bitrate();

	return ((8 * _size) / br);
}

// returns the MPEG version number (1.0 or 2.x);
float mp3info::version()
{
	return _versions[ bits["Version"] ];
}

// returns the MPEG layer number
int mp3info::layer()
{
	return ( 4 - bits["Layer"]);
}

// returns the bitrate in bps
int mp3info::bitrate()
{
	return (int)(_bitrates[ bits["Version"] & 1 ][ bits["Layer"] - 1 ][ bits["Bitrate"] ]) * 1000;
}

// returns the number of frames
int mp3info::frames()
{
	int val = (bits["Layer"] == 3 ? 12 : 144);
	float freq = (val * bitrate() / frequency() + bits["Padding"]);

	return (int)(size()/freq);
}

// true if private, else false
bool mp3info::is_private()
{
	return bits["Private"];
}

// returns stereo mode
char* mp3info::mode()
{
	char* s;

	switch(bits["Mode"])
	{
	case 1:
	        s = "Joint Stereo";
	        break;
	case 2:
	        s = "Dual Channel";
	        break;
	case 3:
	        s =  "Single Channel";
	        break;
	default:
	        s =  "Stereo";
	}

	return s;
}

// true if copyrighted, else false
bool mp3info::copyright()
{
	return bits["Copyright"];
}

// returns the frequency (also known as Samplerate)
int mp3info::frequency()
{
	return _frequency[ bits["Version"] ][ bits["Frequency"] ];
}

// true if CRC protected, else false
bool mp3info::is_protected()
{
        return !bits["Protection"];
}

// true if original, else false
bool mp3info::original()
{
        return bits["Original"];
}

// returns size in bytes of file
long mp3info::size()
{
	return _size;
}




/***********************************************************************
        PRIVATE
 ***********************************************************************/

// initialize variables used by all the constructors
void mp3info::initialize()
{
        if(!file)
        {
                cerr << "Error: unable to open file" << endl;
                exit(1);
        }

	// read the first four bytes, all the mp3 info is there
	file->read(szbuff,4);

	DWORD info = (DWORD)
	(
		((szbuff[0] & 255) << 24) |
		((szbuff[1] & 255) << 16) |
		((szbuff[2] & 255) << 8 ) |
		((szbuff[3] & 255)      )
	);

	// extract the bits and get the info
        bits["FrameSync"]= EXTRACT_BITS(info, 22, 11);
        bits["Version"]	= EXTRACT_BITS(info, 20, 2);
        bits["Layer"] = EXTRACT_BITS(info, 18, 2);
	bits["Protection"] = EXTRACT_BITS(info, 17, 1); // CRC protection
	bits["Bitrate"]	= EXTRACT_BITS(info, 13, 4);
	bits["Frequency"] = EXTRACT_BITS(info, 11, 2);
	bits["Padding"]	= EXTRACT_BITS(info, 10, 1);
	bits["Private"]	= EXTRACT_BITS(info, 9, 1);
	bits["Mode"] = EXTRACT_BITS(info, 7, 2);
	bits["ModeExt"]	= EXTRACT_BITS(info, 5, 2);
        bits["Copyright"] = EXTRACT_BITS(info, 4, 1);
        bits["Original"] = EXTRACT_BITS(info, 3, 1);
        bits["Emphasis"] = EXTRACT_BITS(info, 1, 2);

	_size = get_size();
}

// returns size in bytes of file
long mp3info::get_size()
{
	long len;

	file->seekg (0, ios::end);
	len = file->tellg();
	file->seekg (0, ios::beg);

	return len;
}
